
/* data types */
#define TYPE_CHAR    0x01
#define TYPE_SHORT   0x02
#define TYPE_INT     0x03
#define TYPE_LONG    0x04

#define TYPE_UCHAR   0x05
#define TYPE_USHORT  0x06
#define TYPE_UINT    0x07
#define TYPE_ULONG   0x08

#define TYPE_POINTER 0x09 
        /* pointer type & FFFF0000 : e.g. TYPE_POINTER & (TYPE_CHAR << 16 ) */



/*    F I E L D     */
typedef struct db_field {
   int id;           /* typhoon db ID of field */
   char  *name;      /* name of field */
   int type;         /* data type */
   int size;         /* # of "type" elements: >1 == array of type */
   int index;        /* ID of index for this field; NULL = not indexed */
   int table;        /* ID of table this field references; NULL = not ID */
   int offset;       /* offset from start of record structure */
   char *help;       /* description of field */
   struct db_field *next;
} db_field;


/*    I N D E X     */
typedef struct db_index {
   int id;           /* typhoon ID of index */
   char *name;       /* ASCII name of index */
   db_field *key1;   /* primary key in index (req) */
   db_field *key2;   /* secondary key in index (opt) */
   void *table;      /* pointer to db_table for this index */
   struct db_index *next;
} db_index;


/*    T A B L E     */
typedef struct db_table {
   int id;           /* typhoon ID of table */
   char *name;       /* ASCII name of table */
   int size;         /* size of record structure */
   db_field *fields; /* linked list of fields */
   db_index *index;  /* linked list of indexes */
   struct db_table *next;   
} db_table;


/*    S C H E M  A  */
typedef struct db_schema{
   int id;           /* bastard ID for db */
   char *name;       /* ASCII name of schema, e.g. "target" */
   db_table *tables; /* linked list of tables in schema */
   struct db_schema *next;
} db_schema;


#define REL_NOT      0x01  /* set by user */
#define REL_EQUAL    0x02
#define REL_GREATER  0x04
#define REL_LESS     0x08

#define QUERY_FIRST  0x01  /* set by search function */
#define QUERY_LAST   0x02
#define QUERY_MIN    0x03
#define QUERY_MAX    0x04
#define QUERY_LIST   0x05

typedef struct db_query {
   db_table *table;
   db_index *index;        /* index to use -- optional */
   db_field *field;
   void *value;            /* field must match this value */
   short rel;              /* relation */
   short flag;             /* first, last, min, max, list */
   struct db_query *next;  /* for mutliple conditions */
} db_query;

typedef struct db_result {
   void *data;
   db_table *table;
   struct db_result *next;
} db_result;

db_schema * db_schema_by_name( char *name);
db_table  * db_table_by_name(db_schema *s, char *name);
db_index  * db_index_by_name(db_table *t, char *name);
db_field  * db_field_by_name(db_table *t, char *name);

db_result * db_table_get(db_table *table);
void db_result_free(db_result *r);
void * db_result_data(db_result *r);
int db_result_foreach(db_result *r, void *func, void *data);
int db_result_add(db_result *list, db_result *r);
db_result * db_result_next(db_result *r);
db_result * db_result_prev(db_result *r);
db_result * db_result_head(db_result *r);
db_result * db_result_tail(db_result *r);

int sprint_record( char *buf, int len, char delim, char *table, void *record);
int sprint_field(char *buf, int len, db_field *field, void *record);
/* diagnostic functions */
int print_record( char *table, void *record);
int print_field(db_field *field);
int print_index(db_index *index);
int print_table(db_table *table);
int print_db(void);
