#include <api/api_main.h>
#include <extension.h>
#include <vm.h>

extern struct DISASM_TGT target;
extern struct DISASM_ENV disasm_env;
extern struct DISASM_PREFS disasm_prefs;

/* API sections:
 *     6. Code                  :: Creating/manipulating CODE objects
 * /


/* ---------------------------------------------------------------Code */
/* TODO : rewrite these to use curr_rec like api_search.c does */
int code_new(long rva, struct code *c){
   struct address a;
   
   if (! db_index_find(ADDRESS_RVA, &rva, &a)) {
      return(sys_set_lasterr(4020));
   }
   if (! a.flags & ADDR_CODE){
      a.flags ^= ADDR_DATA;
      a.flags &= ADDR_CODE;
      addr_set_flags(rva, a.flags);
   }
   /* TODO :cehck if exists? Maybe just add DB restraint */
   db_record_insert(CODE, c);
   return(1);
}

int code_del(long rva){
   int *state, flags;

   state = db_save_state();
   flags = addr_flags( rva );
   db_restore_state(state);

   if (flags & ADDR_CODE) {
      /* remove CODE flag from ADDRESS */
      flags ^= ADDR_CODE;
      flags &= ADDR_DATA;
   } else return(1);

   /* TODO: clean up any xrefs and such here !!! */
   return(db_record_delete(CODE_RVA, &rva));
}

int code_set_func(long rva, long func_id){
   struct code c;

   if (db_index_find(CODE_RVA, &rva, &c)) {
      c.func = func_id;
      db_record_update(CODE_RVA, &rva, &c);
   }
   return(0);
}

int code_sprint(char *buf, char *fmt, long rva) {
   struct address a;

   if (d_keyfind(CODE_RVA, &rva) != S_OKAY) 
      return(sys_set_lasterr(4110));
   d_recread(&a);
   return(asmsprintf(buf, fmt, &a));
}


/* ---------------------------------------------------------ADD_EXPR */
/* these routines deal with address expressions. If an operand is of type
 * OP_ADDREXP, then its operand field references an ID in the ADDR_EXP
 * table.
 */

int addrexp_new(int scale, int index, int base, int disp, int flags){
   struct addr_exp a = {0};

   //if (! flags) return(0); /* flags should never be 0 */
   //if (! flags) printf("flags are 0!\n");; /* flags should never be 0 */
   a.scale = scale;
   a.index = index;
   a.base = base;
   a.disp = disp;
   a.flags = flags;
   a.id = ++seq.addr_exp;
   if (db_record_insert(ADDR_EXP, &a)) return(a.id);
   return(0);
}

int addrexp_get( int id, struct addr_exp *exp ){
   return(db_index_find(ADDR_EXP, &id, exp));
}

int addrexp_format_op( int operand, int flags, char *buf, int len){
   /* TODO : fix sign */
   if ( ! operand  && flags != ADDREXP_REG ) {
      buf[0] = '\0';
      return(0);
   }
   switch (flags) {
      case ADDREXP_REG:
         strncpy(buf, GetRegMnemonic(operand), len);
         break;
      case ADDREXP_WORD:
         if (operand) snprintf(buf, len, "%04X", operand);
         break;
      case ADDREXP_DWORD:
         if (operand) snprintf(buf, len, "%08X", operand);
         break;
      case ADDREXP_QWORD:
         if (operand) snprintf(buf, len, "%012X", operand);
         break;
      case ADDREXP_BYTE:
      default:
         if (operand) snprintf(buf, len, "%02X", operand);
   }

   return(strlen(buf));
}
int addrexp_get_str( int id, char *string, int len){
   struct addr_exp exp;
   char ops[32], opi[32], opb[32], opd[32];
   int sign = 0;

   if (db_index_find(ADDR_EXP_ID, &id, &exp)) {
      /* add NULLs for empty values */
      /* note that all register IDs are >= 0 */
      if (exp.scale < 0) sign &= 0x1000; // probably not ever negative
      if (exp.index < 0) sign &= 0x0100;
      if (exp.base  < 0) sign &= 0x0010; // ditto
      if (exp.disp  < 0) sign &= 0x0001;

      /* do scale */
      addrexp_format_op( exp.scale, AddrExp_ScaleType(exp.flags), ops, 32);
      /* do index */
      addrexp_format_op( exp.index, AddrExp_IndexType(exp.flags), opi, 32);
      /* do byte */
      addrexp_format_op( exp.base, AddrExp_BaseType(exp.flags), opb, 32);
      /* do disp */
      addrexp_format_op( exp.disp, AddrExp_DispType(exp.flags), opd, 32);
      /* call assembler for address expression format */
      ext_sprint_addrexp(string, len, ops, opi, opb, opd, sign);
      return(strlen(string));
   }
   else printf("not found!\n");
   return(0);
}

int code_opcmp( long op1, int flag1, long op2, int flag2 ) {
   struct addr_exp *e1, *e2;

   if ( flag1 & 0xFFFF00 != flag2 & 0xFFFF00 ) /* re don't care about perm */
      return(0);
   if ( flag1 & OP_TYPE_MASK != OP_EXPR )
      return( (op1 == op2) ? 1 : 0 );
   /* OK, the operand is an ADDR_EXPR, better compare it */
   e1 = GetAddrExpObject(op1);
   e2 = GetAddrExpObject(op2);

   if (e1 && e2 ) {
      if (e1->flags != e2->flags) return(0);
      if (e1->scale != e2->scale) return(0);
      if (e1->index != e2->index) return(0);
      if (e1->base != e2->base) return(0);
      if (e1->disp != e2->disp) return(0);
      return(1);
   }

   if (e1) free(e1);
   if (e2) free(e2);
   return(0);
}

int code_effect_new(long rva, int reg, int change){
   struct code_effect e;

   e.rva = rva;
   e.reg = reg;
   e.change = change;
   e.id = ++seq.code_effect;
   return(db_record_insert(CODE_EFFECT, &e));
}
