/* api_system.c : part of The Bastard disassembly environment 
 *
 */
#include <ctype.h>
#include <dirent.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>

#include <api/api_main.h>
#include <api/api_error.h>
#include <extension.h>
#include <config.h>

extern struct DISASM_TGT target;
extern struct DISASM_ENV disasm_env;
extern struct DISASM_PREFS disasm_prefs;

/*    API section:
 *
 *    17. System Interaction   :: Various utility functions
 * /

/* ------------------------------------------------------System Interaction */
void sys_init(char *home) {
   env_defaults();
   DEBUG_Print("sys_initializing Interpreter\n");
   sys_initInterpreter();
   DEBUG_Print("sys_initializing RDBMS\n");
   db_init();
   /* set up DB linked list */
   db_head = db_current = db_target;
   db_target->db_id = -1;
   /*DEBUG_Print("Loading Config DB\n");
   db_head = db_current = db_target->prev = db_config;
   db_config->next = db_target;
   db_config->db_id = db_target->db_id = -1;

   LoadConfigDB(NULL);*/
   DEBUG_Print("sys_initializing Extensions\n");
   sys_initExtensions();
   sys_re_init(home);
   return;
}

void sys_re_init(char *home) {
   char *user, name[PATH_MAX];
   
   
   DEBUG_Print("Resetting Environment\n");
   /* set all structs to 0 and set defaults */
   memset(&target, 0, sizeof(struct DISASM_TGT));
   env_defaults();
   env_target_defaults();
   env_pref_defaults();

   /* set up pseudovirtualmachine */
   DEBUG_Print("sys_initializing VM\n");
   sys_initVM();

   /* load default extensions */
   DEBUG_Print("Loading Default Extensions\n");
   sprintf(name, "%s/arch/lib%s.so", disasm_env.home, target.arch);
   DEBUG_PrintVar( name, "%s");
   DEBUG_PrintVar( disasm_env.home, "%s");
   DEBUG_PrintVar( target.arch, "%s");
   LoadExtension( EXT_ARCH, name, ext_arch);
   sprintf(name, "%s/asm/lib%s.so", disasm_env.home, target.assembler);
   DEBUG_PrintVar( name, "%s");
   DEBUG_PrintVar( target.assembler, "%s");
   LoadExtension( EXT_ASM, name, ext_asm);
   sprintf(name, "%s/formats/%s.bc", disasm_env.home, target.format);
   DEBUG_PrintVar( name, "%s");
   DEBUG_PrintVar( target.format, "%s");
   LoadExtension( EXT_FORMAT, name, ext_format);
   sprintf(name, "%s/lang/lib%s.so", disasm_env.home, target.lang);
   DEBUG_PrintVar( name, "%s");
   DEBUG_PrintVar( target.lang, "%s");
   LoadExtension( EXT_LANG, name, ext_lang);

   /* load history for previous user */
   DEBUG_Print("Loading User Profile\n");
   user = getenv("USER");
   //LoadProfile(PROFILE_ENV, user);
   //LoadProfile(PROFILE_PREF, user);

   /* override everything with command-line options etc */
   disasm_prefs.options |= disasm_prefs.override;
   if ( disasm_prefs.options & COLOR_TTY) 
      disasm_env.output = ASM_OUTPUT_TTY_COLOR;
   
   return;
}
	
	
int sys_find_home(char *buf, int len, char **argv){
	int x;
	FILE *fTemp;
	char tmpBuf[128], *tmp;
	struct stat tmpStat;
	
	/* + Get the location of bastard [for .bdb, libs, etc] */
	/* the key to our patented 'Run Anywhere' technology ;) */
	if ( !(tmp = getenv("BASTARD_HOME")) ){
   	tmp = tmpBuf;
   	if (strchr(argv[0], '/') != NULL) {
      	strncpy(tmp, argv[0], sizeof(tmpBuf) -1);
   	} else {
      	fTemp = popen("which bastard", "r");
      	if (! fTemp ) {
         	printf("Which is broken! Use Explicit Path Names!\n");
         	return(0);
      	} else {
         	fgets(tmp, sizeof(tmpBuf), fTemp);
         	pclose(fTemp);
         	x = strlen(tmp) - 1;
         	while( isspace(tmp[x]) ) {
            	/* strip carriage return and other nonsense */
            	tmp[x] = 0;
            	x--;
         	}
      	}
   	}
   	/* handle symbolic links to the bastard */
   	if (lstat( tmp, &tmpStat) == -1) {
      	printf("Fatal error %s: lstat %s\n",strerror(errno), tmp);
      	return(0);
   	}
   	if (S_ISLNK(tmpStat.st_mode)){
      	if (readlink( tmp, tmp, 128) == -1) {
          	printf("Fatal Error %X: readlink %s\n", errno, tmp);
         	return(0);
      	}
   	} 
	}
	
	if (len < strlen(tmp)) {
		printf("Fatal Error : %d byte buffer is too small for path '%s'\n", 
				len, tmp);		
		return(0);
	}
	
	strncpy(buf, tmp, (strrchr(tmp, '/') - tmp )); /*get rid of trailing / */
	/* NOTE: all binaries must be in $BASTARD_HOME/bin for this to work */
	strcat(buf, "/..");
	return(1);
	
}		
void sys_msg( const char *str) {
/* Print a message unless QUIET is flag is set 
 */
   if (! (disasm_prefs.options & QUIET))  printf("%s", str);
   return;
}

void sys_errmsg( int num, const char *str) {
/* Print a message and error code, regardless of QUIET */
   printf("Error %d: %s\n", num, str);
   return;
}

void sys_quit(void) {
   char *user = getenv("USER");

   DEBUG_Print("sys_quitting...\n");
   /* disabled until flags are working everywhere 
     if ( disasm_env.flags )  */
   if (target.image){
      target_close_db();
   }
   /* DEBUG_Print("Updating user profile\n");
   //UpdateProfile(PROFILE_ENV, user);
   //UpdateProfile(PROFILE_PREF, user);
   UnloadConfigDB(); */
   CleanupVM(); 
   DEBUG_Print("Cleaning Up Extensions\n");
   CleanupExtensions();
   env_set_flag( QUIT_NOW );
}

int sys_exec( char *args ){
   char buf[256];
   sprintf( buf, ".bdb/%s %s", target.name, args); 
   return(system(buf));
}

int sys_debug(void) {
   char buf[256];
   sprintf( buf, "%s .bdb/%s", disasm_prefs.debugger, target.name); 
   return(system(buf));
}

int sys_core(void) {
/* start target in new process w/ core dumps enabled */
/* wait for user to hit key */
/* send a sig_core to target PID */
   sys_msg("sys_core not yet supported!\n");
   return(-1);
}

int sys_panic(void) {
/* Similar to 'core' */
/* Get a kernel dump w/o killing the kernel, if possible */
/* ... the idea is to get a snapshot of the kernel as a target is
 * running */
   sys_msg("sys_panic not yet supported!\n");
   return(-1);
}

/* ----------------------------------------------- Error Handling */
int _last_bastard_error;

int sys_set_lasterr( int errnum ){
/* This returns 0 so that you can do a `return(sys_set_lasterr(#));` */
   _last_bastard_error = errnum;
   return(0);
}

char* sys_get_errstr( int errnum ){
   int x;

   for (x=0; _bastard_error_table[x].errnum != 0; x++)
      if (_bastard_error_table[x].errnum == errnum)
         return( _bastard_error_table[x].errstr);
   return( NULL ) ;   
}

int sys_get_lasterr(){
   return( _last_bastard_error );
}

char* sys_lasterr_str( ){
   return( sys_get_errstr( _last_bastard_error) );
}

void sys_print_errmsg(int errnum){
   char *msg;

   sys_errmsg( errnum, sys_get_errstr(errnum));
   return;
}
void sys_sprint_errmsg(char *buf, int errnum){
   if (! buf) 
      return;
   strcpy( buf, sys_get_errstr(errnum) );
   return;
}


