
/* DB interface for the bastard + typhoon */

#include <bastard.h>
#include <bdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "db/db_query.h"
#include "db/db_def.h"



int sprint_field_value(char *buf, int len, int type, void *data, int size ){
   int rv = 0;

   switch (type){
      case TYPE_CHAR:
         if ( size > 1)
            rv=snprintf(buf, len, "%s", (char *)data);
         else
            rv=snprintf(buf, len, "%c", *(char *)data);
         break;
      case TYPE_SHORT:
         rv=snprintf(buf, len, "%d", *(short *)data); break;
      case TYPE_INT:
         rv=snprintf(buf, len, "%d", *(int *)data); break;
      case TYPE_LONG:
         rv=snprintf(buf, len, "0x%08lX", *(long *)data); break;
      case TYPE_UCHAR:
         rv=snprintf(buf, len, "%c", *(unsigned char *)data); break;
      case TYPE_USHORT:
         rv=snprintf(buf, len, "%d", *(unsigned short *)data); break;
      case TYPE_UINT:
         rv=snprintf(buf, len, "%d", *(unsigned int *)data); break;
      case TYPE_ULONG:
         rv=snprintf(buf, len, "0x%08lX", *(unsigned long *)data); break;
      case TYPE_POINTER:
         rv=snprintf(buf, len, "0x%08X", data); break;
      }
   return(rv);
}

int print_field(db_field *field){
   switch (field->type){
      case TYPE_CHAR:
         printf("\t\tchar"); break;
      case TYPE_SHORT:
         printf("\t\tshort"); break;
      case TYPE_INT:
         printf("\t\tint"); break;
      case TYPE_LONG:
         printf("\t\tlong"); break;
      case TYPE_UCHAR:
         printf("\t\tunsigned char"); break;
      case TYPE_USHORT:
         printf("\t\tunsigned short"); break;
      case TYPE_UINT:
         printf("\t\tunsigned int"); break;
      case TYPE_ULONG:
         printf("\t\tunsigned long"); break;
   }
   printf(" %s", field->name);
   if (field->size > 1) printf("[%d]", field->size);
   printf("\n");
   return(1);
}
int print_index(db_index *index){
   printf("\t\t%s { %s", index->name, index->key1->name);
   if (index->key2) printf(", %s", index->key2->name);
   printf(" }\n");
   return(1);
}

int sprint_field( char *buf, int len, db_field *field, void *record){
   char *data = (char *) record + field->offset;
   return ( sprint_field_value(buf, len, field->type, data, field->size));
}

int sprint_record( char *buf, int len, char delim, char *table, void *record) {
   db_table *t;
   db_field *fld;
   char  *data;
   char *fmt_str, tmp[256];

   t = target_schema;
   while (t) {
      if (! strcmp(table, t->name)) break;
      t = t->next;
   }
   if (! t) return(0);

   fld = t->fields;
   while (fld) {
      memset(tmp, 0, 256);
      (char *)data = (char *) record + fld->offset;
      sprint_field_value(tmp, 256, fld->type, data, fld->size);
      tmp[strlen(tmp)] = delim;
      strncat(buf, tmp, len);
      
      fld = fld->next;
   }
   /* get rid of last delim */
   len = strlen(buf);
   buf[len - 1] = 0;
   len--;

   return(len);
}

int print_record( char *table, void *record){
   db_table *t;
   db_field *fld;
   long *rec = record, *data;
   char *fmt_str, buf[256];

   t = target_schema;
   while (t) {
      if (! strcmp(table, t->name)) break;
      t = t->next;
   }
   if (! t) return(0);

   fld = t->fields;
   while (fld) {
      (char *)data = (char *) rec + fld->offset;
      sprint_field_value(buf, 256, fld->type, data, fld->size);
      printf("%s\n", buf);
      fld = fld->next;
   }
   printf("\n");
   return(1);
}
int print_table(db_table *table){
   db_field *fld;
   db_index *idx;

   printf("\tTable %s\n", table->name);
   printf("\t  Record:\n");
   fld = table->fields;
   while (fld) {
      print_field(fld);
      fld = fld->next;
   }
   printf("\n");

   printf("\t  Indexes:\n");
   idx = table->index;
   while (idx) {
      print_index(idx);
      idx = idx->next;
   }
   printf("\n");
   return(1);
}

int print_db(void){
   db_table *current = target_schema;

   printf("Database TARGET\n");
   while (current) {
      print_table(current);
      current = current->next;
   }
   printf("EOF\n");
   return(1);
}

int db_table_foreach(db_table *table, void *func, void *arg){
   /* return number of foreach iterations */
   return(0);
}

/* schema management */


db_schema * db_schema_by_name( char *name) {
   db_schema *s = all_schemas;
   
   while( s ) {
      if (! strcmp(s->name, name)) return(s);
      s = s->next;
   }
   return(NULL);
}



/* table management *//* head - foot - min -max - count */

db_table * db_table_by_name(db_schema *s, char *name){
	db_table *t = s->tables;
	
	while ( t ) {
		if (! strcmp(t->name, name)) return(t);
		t = t->next;
	}
	return(NULL);
}

db_index * db_index_by_name(db_table *t, char *name){
	db_index *i = t->index;
	
	while ( i ) {
		if (! strcmp(i->name, name)) return(i);
		i = i->next;
	}
	return(NULL);
}

db_field * db_field_by_name(db_table *t, char *name){
	db_field *f = t->fields;
	
	while ( f ) {
		if (! strcmp(f->name, name)) return(f);
		f = f->next;
	}
	return(NULL);
}

/* use this to get an LL for SECTION etc */
db_result * db_table_get(db_table *table) {
   /* return LL of results structs (ptr to actual structs */
	db_result *r, head = {0};
	char *data;
	int cont;
	
	r = &head;
	data = calloc(table->size, 1);	/* saves us a memcpy each time */	
	cont = db_index_first(table->index->id, data);

	while (cont) {
		r->next = calloc(sizeof(db_result), 1);
		r = r->next;
		r->data = data;
		r->table = table;
		
		data = calloc(table->size, 1);
		cont = db_index_next(table->index->id, data);
	}
	
	free(data);		/* the last calloc will be unused */
   return(head.next);
}

/* searching */
int db_field_cmp(db_field *f, void *data, int rel){
	
	return(0); /* FALSE */
}

db_result * db_find_indexed(db_query *q){
	/* while keyfind/keynext in index, if db_field_cmp, add to results */
	return(NULL);
}

db_result * db_find_unindexed(db_query *q){
	/* for i in table */
	/*   if db_field_cmp, add to results */
	return(NULL);
}

db_result * db_select(void *query){
	return(NULL);
}
/* inserting */
/* updating */
/* deleting */



/* db_result management */
void db_result_free(db_result *r){
	db_result *i, *j;
	
	i = r;
	while (i) {
		if (i->data) free(i->data);
		j = i->next;
		free(i);
		i = j;
	}
   return;
}
void * db_result_data(db_result *r){
   return(NULL);
}
int db_result_foreach(db_result *r, void *func, void *data){
   return(0);
}
int db_result_add(db_result *list, db_result *r){
   return(0);
}
db_result * db_result_next(db_result *r){
   return(NULL);
}
db_result * db_result_prev(db_result *r){
   return(NULL);
}
db_result * db_result_head(db_result *r){
   return(NULL);
}
db_result * db_result_tail(db_result *r){
   return(NULL);
}
