#include <glob.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <bastard.h>

#include "support.h"
#include "db_browse.h"
#include "db/db_query.h"


extern GtkWidget *sob_main_window;

struct tree_node db_tree = {0};

int add_leaf_node(GtkTreeItem *parent, char *label) {
	GtkTree *t;
	GtkTreeItem *i;
	
	t = gtk_tree_new();
	gtk_tree_item_set_subtree(parent, t);
		
	i = gtk_tree_item_new_with_label(label);
	gtk_tree_append(t, i);	
	gtk_widget_show(i);
	return(0);
}

int init_table_count(struct db_table *table, GtkTree *tree){
	GtkTreeItem *node;

	/* 'count' label should be generated dynamically */
	node = gtk_tree_item_new_with_label("0");
	gtk_tree_append(tree, node);
	gtk_widget_show(node);
	return(0);
}

int init_table_desc(struct db_table *table, GtkTree *tree){
	GtkTreeItem *node;
	db_field *f = table->fields;
	
	while (f) {
		node = gtk_tree_item_new_with_label(f->name);
		/* add type value to node data */
		gtk_tree_append(tree, node);
		gtk_widget_show(node);
		f = f->next;
	}
	return(0);
}

struct DB_TREE_CB_ITEM {
	db_table *t;
	GtkTree *tree;
};

/* Callbacks for DB Tree */
void expand_db_tree_cb(char *record, struct DB_TREE_CB_ITEM *s){
	char buf[256] = {0};
	GtkTreeItem *item;	

	sprint_record(buf, 256, '|', s->t->name, record);
	item = gtk_tree_item_new_with_label(buf);
	gtk_tree_append(s->tree, item);
	gtk_widget_show(item);
	return;
}

void expand_db_tree(GtkTreeItem *node, db_index *i){
	struct DB_TREE_CB_ITEM s;
	
	s.t = i->table;
	s.tree = GTK_TREE_ITEM_SUBTREE(node);
	
	db_index_foreach(i->id, &expand_db_tree_cb, &s);
	return;
}
void expand_db_count(GtkTreeItem *node, db_table *t){
	/* set leaf node data */
	GtkTree *tree;
	GtkTreeItem *item;
	char buf[32] = {0};
	
	sprintf(buf, "%d Records", db_table_count(t->id));
	tree = GTK_TREE_ITEM_SUBTREE(node);
	item = gtk_tree_item_new_with_label(buf);
	gtk_tree_append(tree, item);
	gtk_widget_show(item);
	
	return;
}

void expand_db_min(GtkTreeItem *node, db_index *i){
	/* set leaf node data */
	GtkTree *tree;
	GtkTreeItem *item;
	db_table *t = i->table;
	char buf[256] = {0};
	char *record = calloc(t->size, 1);
	
	db_index_first(i->id, record);	
	sprint_record(buf, 256, '|', t->name, record);
	tree = GTK_TREE_ITEM_SUBTREE(node);
	item = gtk_tree_item_new_with_label(buf);
	gtk_tree_append(tree, item);
	gtk_widget_show(item);
	
	free(record);
	return;
}

void expand_db_max(GtkTreeItem *node, db_index *i){
	/* set leaf node data */
	GtkTree *tree;
	GtkTreeItem *item;
	db_table *t = i->table;
	char buf[256] = {0};
	char *record = calloc(t->size, 1);
	
	db_index_last(i->id, record);
	sprint_record(buf, 256, '|', t->name, record);
	tree = GTK_TREE_ITEM_SUBTREE(node);
	item = gtk_tree_item_new_with_label(buf);
	gtk_tree_append(tree, item);
	gtk_widget_show(item);
	
	free(record);	return;
}

void collapse_db_dyntree(GtkTreeItem *node, db_index *i){
	GtkTree *t;
	gtk_tree_item_remove_subtree(node);	
	
	t = gtk_tree_new();
	gtk_tree_item_set_subtree(node, t);
	return;
}

int init_table_index(struct db_table *table, GtkTree *tree){
	GtkTreeItem *node;
	GtkTree *idx_tree, *t;
	db_index *i = table->index;
	
	while (i) {
		node = gtk_tree_item_new_with_label(i->name);
		/* add type value to node data */
		gtk_tree_append(tree, node);
		gtk_widget_show(node);
		
		/* make subtree for index */
		idx_tree = gtk_tree_new();
		gtk_tree_item_set_subtree(node, idx_tree);
		
		/* Add Browse Node */		
		node = gtk_tree_item_new_with_label("Browse");
		/* attach signal for user clicking on 'browse' node */   
      gtk_signal_connect (GTK_OBJECT (GTK_TREE_ITEM (node)), "expand",
            (GtkSignalFunc) expand_db_tree, i );
		gtk_signal_connect (GTK_OBJECT (GTK_TREE_ITEM (node)), "collapse",
            (GtkSignalFunc) collapse_db_dyntree, i );
		gtk_tree_append(idx_tree, node);
		gtk_widget_show(node);
		t = gtk_tree_new();
		gtk_tree_item_set_subtree(node, t);
				
		/* Add Max Node */		
		node = gtk_tree_item_new_with_label("Max");
      gtk_signal_connect (GTK_OBJECT (GTK_TREE_ITEM (node)), "expand",
            (GtkSignalFunc) expand_db_max, i );
      gtk_signal_connect (GTK_OBJECT (GTK_TREE_ITEM (node)), "collapse",
            (GtkSignalFunc) collapse_db_dyntree, i );
	  	gtk_tree_append(idx_tree, node);
		gtk_widget_show(node);
		t = gtk_tree_new();
		gtk_tree_item_set_subtree(node, t);

		/* Add Min Node */		
		node = gtk_tree_item_new_with_label("Min");
		gtk_signal_connect (GTK_OBJECT (GTK_TREE_ITEM (node)), "expand",
            (GtkSignalFunc) expand_db_min, i );
      gtk_signal_connect (GTK_OBJECT (GTK_TREE_ITEM (node)), "collapse",
            (GtkSignalFunc) collapse_db_dyntree, i );
		gtk_tree_append(idx_tree, node);
		gtk_widget_show(node);
		t = gtk_tree_new();
		gtk_tree_item_set_subtree(node, t);
		
		/* Add Primary Key node */		
		node = gtk_tree_item_new_with_label("Primary Key");
		gtk_tree_append(idx_tree, node);
		add_leaf_node(node, i->key1->name);
		gtk_widget_show(node);
		
		/* Add Secondary Key node */		
		node = gtk_tree_item_new_with_label("Secondary Key");
		gtk_tree_append(idx_tree, node);
		if (i->key2) add_leaf_node(node, i->key2->name);
		gtk_widget_show(node);
		
		i = i->next;
	}
	return(0);
}

int init_db_tree(){
	GtkTree *tree;
	GtkTreeItem *node;
	struct db_table *table;
	struct db_schema *s;
	struct tree_node *t = &db_tree;
	GtkTree *db_root = (GtkTree *) lookup_widget(sob_main_window, "dbTree");
	
	db_tree.label = "Target";
	s = db_schema_by_name("TARGET");
	for (table = s->tables; table; table = table->next) {
		/* 1. add node for table to root */
		t->next = calloc(sizeof(struct tree_node), 1);
		t = t->next;
		t->label = table->name;
		t->parent = db_root;
		t->node = gtk_tree_item_new_with_label(table->name);
		gtk_tree_append(db_root, t->node);
		
		/* 2. Create subtree for table */
		t->subtree = gtk_tree_new();
		gtk_tree_item_set_subtree( t->node, t->subtree);

		/* 3. Add COUNT subtree */
		node = gtk_tree_item_new_with_label("Count");
		gtk_tree_append(t->subtree, node);
		//tree = gtk_tree_new();
		gtk_signal_connect (GTK_OBJECT (GTK_TREE_ITEM (node)), "expand",
            (GtkSignalFunc) expand_db_count, table );
      gtk_signal_connect (GTK_OBJECT (GTK_TREE_ITEM (node)), "collapse",
            (GtkSignalFunc) collapse_db_dyntree, table );
		//gtk_tree_item_set_subtree(node, tree);
		//init_table_count(table, tree);
		gtk_widget_show(node);
		tree = gtk_tree_new();
		gtk_tree_item_set_subtree(node, tree);
	
		
		/* 4. Add DESC subtree */
		node = gtk_tree_item_new_with_label("Desc");
		gtk_tree_append(t->subtree, node);
		tree = gtk_tree_new();
		gtk_tree_item_set_subtree(node, tree);
		init_table_desc(table, tree);
		gtk_widget_show(node);
		
		/* 5. Add INDEX subtree */
		node = gtk_tree_item_new_with_label("Index");
		gtk_tree_append(t->subtree, node);
		tree = gtk_tree_new();
		gtk_tree_item_set_subtree(node, tree);
		init_table_index(table, tree);
		gtk_widget_show(node);
		
		gtk_widget_show(t->node);
	}
	return(1);			
}

