#include <stdio.h>
#include <stdlib.h>
#include <bastard.h>
#include <bdb.h>

#include "sob_asm_win.h"
#include "support.h"


extern GtkWidget *sob_main_window; /* global main-window definition */

struct DISPLAY asm_display;

int total_db_lines; /* global # of addresses in DB */

void init_asm_win(){
	int x;
	
	/* Prepare Display Window data */
	/* TODO: replace w/ get window size */
	asm_display.size = 26; 
	asm_display.init = 0;
	asm_display.upper = 1000;
	asm_display.lower = 0;
	
	/* Prepare Display Buffer */
	asm_display.buf.size = 256;
	asm_display.buf.line_size = 128;
	asm_display.buf.lines = calloc(sizeof(struct DISP_LINE), asm_display.buf.size);
	asm_display.buf.start = asm_display.buf.end = 0;
		
	for (x=0; x < asm_display.buf.size; x++){
		asm_display.buf.lines[x].text = calloc(asm_display.buf.size, 1);
		if (x + 1 < asm_display.buf.size) 
			asm_display.buf.lines[x].next = &asm_display.buf.lines[x+1];
		else
			asm_display.buf.lines[x].next = NULL;		
	}
	
	/* Prepare Window Widget */
	init_asm_display();	
}

void free_asm_disp_buffer(){
	free(asm_display.buf.lines);
}
	
int get_addr_index_of_rva(long rva){
	struct address a;
	int x, cont;
	
	cont = db_find_closest_prev(ADDRESS_RVA, &rva, &a);
	for (x = 0; cont; x++ ){
		cont = db_index_prev(ADDRESS_RVA, &a);
	}
	return(x);
}

unsigned long get_first_addr(){
	struct address a = {0};
	
	db_index_first(ADDRESS_RVA, &a);
	return(a.rva);
}
unsigned long get_last_addr(){
	struct address a={0};
	
	db_index_last(ADDRESS_RVA, &a);
	return(a.rva);	
}

int get_total_lines(){
	struct address a;
	int x, cont;
	
	cont = db_index_first(ADDRESS_RVA, &a);
	for (x = 0; cont; x++){
		cont = db_index_next(ADDRESS_RVA, &a);
	}
	
	total_db_lines = x;
	
	//printf("first addr: %08x\n", get_first_addr());
	//printf("last addr: %08x\n", get_last_addr());
	return(x);
}

long get_rva_from_offset(long rva, int offset){
	/* This routine finds 'rva' in the database, then iterates through
	   the DB until it finds the ADDRESS record that is 'diff'
		records away from 'rva'; it returns the rva of that record. */	
	int (*db_find)(int, void *, void *);
   int (*db_next)(int, void *);
	struct address a;
	int x, cont, num_addr;
	long ret_rva = rva;
	
	if (offset > 0) {
		db_find = db_find_closest_next;
		db_next = db_index_next;
		num_addr = offset;
	} else if (offset == 0) {
		/* just in case something goes stupid on us */
		return(rva);
	} else {
		db_find = db_find_closest_prev;
		db_next = db_index_prev;
		num_addr = 0 - offset;
	}
	
	/* find the closest address to the one we started with */
	(*db_find)(ADDRESS_RVA, &rva, &a);

	/* iterate through addresses until offset it met */	
	for (x = 0; x < num_addr; x++ ) {
		if ((*db_next)(ADDRESS_RVA, &a)) {
			ret_rva = a.rva;
		} else {
			x = num_addr;
		}
	}
	
	/* done! */
	return(ret_rva);
}

int get_buf_idx_from_rva(struct DISP_BUFFER *buf, long rva){
	/* Look through buffer for the first line containing 'rva'; return the
	   index of that line */
	int x;
	for (x = 0; x < buf->size; x++) {
		if (rva >= buf->lines[x].rva && rva < buf->lines[x + 1].rva) 
			return(x);
	}
	return(-1); /* rva is not in buffer */
}	

void load_display_buf(struct DISPLAY *disp, long addr){
	int x, endtbl = 0;
	struct address a;
	unsigned long rva = addr;
	struct DISP_BUFFER *buf = &disp->buf;
	/* these 2 are temporary, for testing */
	char *code_fmt = "%n%:%a %8b\t%m\t%d%, %s %c %;%x\n";
	char *data_fmt = "%n%:%a %8b\t%c %;%x\n";
	
	//printf("loading display buf %08X \n", rva);
	db_find_closest_prev( ADDRESS_RVA, &rva, &a);
	for (x = 0; x < buf->size; x++){
		memset(buf->lines[x].text, 0, buf->line_size);
		
		if (endtbl) {
		 	buf->lines[x].type = buf->lines[x].rva = 0;
			strcpy(buf->lines[x ].text, "\n");;
		} else {
			//printf("line[%d]: rva %08X\n", x, a.rva);		
			if ( a.flags & ADDR_CODE) { 
				asmsprintf(buf->lines[x].text, code_fmt, &a);
			} else {
				asmsprintf(buf->lines[x].text, data_fmt, &a);
			}
		
			buf->lines[x].rva = a.rva;
			buf->lines[x].type = a.flags; /* TODO fix types :) */
			rva = a.rva + a.size;	/* next rva to print */
			if ( !( db_find_closest_next(ADDRESS_RVA, &rva, &a)) ){
				endtbl = 1;
			}
		}
	}
	buf->start = buf->lines[0].rva;
	buf->end = buf->lines[buf->size-1].rva;
	buf->base = 0;
	//printf("buf start %08lx buf end %08lx\n", buf->start, buf->end);

	return;
}

int load_display_buf_middle(struct DISPLAY *disp, long rva){
	/* This routine loads the display buffer starting with address
	   (rva - (disp->buf.size/2)) so that rva is more or less in
		the center of the buffer. It then returns the buffer index
		of the first line relating to 'rva' */
	int start_rva = get_rva_from_offset(rva, -(disp->buf.size/2));
	
	load_display_buf(&asm_display, start_rva);
	return(get_buf_idx_from_rva(&disp->buf, rva));
}

int load_asm_win_buf(long rva){
	return(load_display_buf_middle(&asm_display, rva));
}


int fill_asm_win( struct DISPLAY *disp, int start) {
	int x;
	struct DISP_BUFFER *buf = &disp->buf;
	GtkText *t = (GtkText *)lookup_widget(sob_main_window, "AsmView");

	if (start < 0) start = 0;

//	printf("filling_asm %d - %d\n", start, start+disp->size);
	gtk_text_set_point( t, 0 );
	gtk_text_forward_delete ( t, gtk_text_get_length( t ) );
	for (x = start; x < start + disp->size; x++ ) {
//		printf("insert line %d rva %x\n", x, buf->lines[x].rva); 
		if (buf->lines[x].text)
			gtk_text_insert( t, NULL, NULL, NULL, buf->lines[x].text, -1);
		else
			/* replace this with BLANK_LINE to preserve bg folor etc */
			gtk_text_insert( t, NULL, NULL,NULL,"\n", -1);
	}

	/* set start/end of DISPLAY [as buffer indexes] */
	disp->start = start;
	disp->end = start + disp->size;
	return(1);
}

int update_asm_win(int old, int new) {
	/* This is called when the scrollbar is used */
	unsigned long rva; 
	struct DISPLAY *disp = &asm_display; /* for convenience */
	int start, diff = new - old;		
	unsigned long first = get_first_addr();
	unsigned long last = get_last_addr();

	if ( new == disp->lower ) {
		/* jump to first addr */
		load_display_buf(disp, first);
		start = 0;
	} else if ( new == disp->upper) { 
		/* jump to last addr */
		start = load_display_buf_middle(disp, last);
//		printf("loaded last addr %08X %d\n", last, start);
	} else if ( diff == 1 || diff == -1 || 
		  diff == disp->size || diff == -(disp->size)){
		/* use 'diff' as number of buffer lines to scroll up/down */
		start = disp->start + diff;
		if ( start < 0 || start + disp->size > disp->buf.size) {
			/* ohshit, this is outside the buffer */
			/* 1. reload buf with current rva in the middle */
			start = load_display_buf_middle(disp,
													  disp->buf.lines[disp->start].rva);
			/* 2. Use diff as offset from this new location */
			start += diff;
		}
		/* at this point we can just use 'start' in a call to fill_asm_win() */
	} else if ( diff != 0 ) {
		/* dammit! user positioned the scrollbar directly! */
		/* 1. convert the diff to an rva */
		rva = get_rva_from_offset(disp->buf.lines[disp->start].rva, diff);
		/* 2. if rva not in buf, reload buf */
		
		if (rva == first){
			start = 0;
			load_display_buf(disp, rva);
		} else if (rva == last) {
			start = disp->buf.size - 1;
			load_display_buf(disp, rva - (disp->size/2));
		} else if ( rva < disp->buf.start || rva + disp->size > disp->buf.end){
			/* Reload buf with this rva in the middle */
			start = load_display_buf_middle(disp, rva);
		/* 3. ...else get line_num of rva in buffer */
		} else {
			start = get_buf_idx_from_rva(&disp->buf, rva);
		}
	} else return(0);
			
	
	/* OK, fill AsmView with new info */
	fill_asm_win( disp, start);
	
	/* set current */
	
	return(1);
}

void calc_asm_win_size(){
	/* This is called after the target had been loaded: its main
	   purpose is to resize the V scrollbar to represent the total
		number of addresses in the target DB ADDRESS table.
	*/
	GtkAdjustment *vadj;
	struct DISPLAY *disp = &asm_display;
	GtkRange *vscroll =(GtkRange *)lookup_widget(sob_main_window, "AsmVScroll");

	/* This resizes the V scrollbar that manages the text window */
	vadj = gtk_range_get_adjustment(vscroll);
	vadj->upper = disp->upper = get_total_lines();
	vadj->lower = disp->lower = 0; 	
	
	/* get starting position of scrollbar (at entry point) */
	disp->y = get_addr_index_of_rva(disp->buf.lines[disp->start].rva);
	gtk_adjustment_set_value(vadj, disp->y);
	gtk_adjustment_changed(vadj);
	gtk_signal_emit_by_name (GTK_OBJECT (vadj), "changed");
	
	vadj = gtk_range_get_adjustment(vscroll);
	//printf("upper: %d\n", vadj->upper);	

	return;
}

int config_asm_win( int start ) {
	struct DISPLAY *disp = &asm_display;
	
	if (start < 0 || start + disp->size > disp->buf.size)
		return(0);
	disp->start = disp->current = start;
	disp->end = start + disp->size - 1;
	
	calc_asm_win_size();
	
	disp->init = 1;
	
	/* OK, we have resized ... now fill with asm */
	fill_asm_win(disp, start);	
	return(1);
}

void init_asm_display(void){
	/* This sets up the AsmView window initially. Specifically, the H/V
	   scrollbars are reconfigured to not be tied to the Text widget.
	*/
	GtkText *AsmView = (GtkText *) lookup_widget(sob_main_window, "AsmView");
	GtkRange *scroll;
	GtkAdjustment *adj;	
	
	/* Setup HScroll*/
	scroll = (GtkRange *) lookup_widget(sob_main_window,"AsmHScroll");
	
	/* why does this not work? */
	adj = AsmView->hadj;
	gtk_range_set_adjustment(scroll, adj);
	asm_display.x = adj->value;
	adj->upper = asm_display.buf.line_size; 	/* default initial value */
	adj->lower = 0; 
	gtk_adjustment_changed(adj);
	gtk_signal_emit_by_name (GTK_OBJECT (adj), "changed");
	gtk_signal_connect (GTK_OBJECT(adj), "value_changed",
                       GTK_SIGNAL_FUNC(cb_hscroll_asmwin), &asm_display.x);
	
	/* Setup VScroll */
	scroll = (GtkRange *) lookup_widget(sob_main_window,"AsmVScroll");
	adj = gtk_adjustment_new( 0, 0, 100, 1,
			                    asm_display.size, asm_display.size);
	/* reset value when buffer is filled! */
	asm_display.y = adj->value;
   gtk_range_set_adjustment(scroll, adj);
	gtk_signal_emit_by_name (GTK_OBJECT (adj), "changed");
	gtk_signal_connect (GTK_OBJECT(adj), "value_changed",
                       GTK_SIGNAL_FUNC(cb_vscroll_asmwin), &asm_display.y);
	
	/* Prepare AsmView Text Widget */
	gtk_text_set_line_wrap( AsmView, 0);

	return;
}
		

void cb_hscroll_asmwin(GtkAdjustment *adj, float *data){	
//	printf("old val: %f new val: %f\n", *data, adj->value);
	*data = adj->value;
}

void cb_vscroll_asmwin(GtkAdjustment *adj, float *data){
	int old, new; /* convert moronic floats to sensible integers */
		
	if (! asm_display.init) return;
	old = *data;
	new = adj->value;
//	printf("old val: %d new val: %d\n", old, new);
	*data = adj->value;
	/* deal with scrollbar action : send old & new positions */
	update_asm_win(old, new);
}
