#include <glob.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <bastard.h>

#include "support.h"
#include "sob_ui.h"
#include "sob_extensions.h"    /* for load dlg */
#include "db_browse.h"         /* for init_ui */
#include "sob_asm_win.h"		/* for init_ui */



GtkWidget *sob_main_window; /* global main-window definition */

/* Global DLG structs */
struct  lfd_dlg load_file_dlg = {0};

int init_load_file_dlg(){
	char default_target[PATH_MAX];	

	
	/* set default target to ./a.out */
	getcwd(default_target, PATH_MAX);
	strcat(default_target, "/a.out");
	 
	gtk_entry_set_text(  
			GTK_ENTRY(lookup_widget( load_file_dlg.self, "lfd_file_entry")),
			default_target ); 

	/* fill structs with widget stuff */
	/* frames */
	load_file_dlg.disasm.frame =  (GtkFrame *)
			lookup_widget( load_file_dlg.self, "disasm_method_frame");
	load_file_dlg.format.frame =  (GtkFrame *) 
			lookup_widget( load_file_dlg.self, "format_opt_frame"); 
	load_file_dlg.cpu.frame =  (GtkFrame *) 
			lookup_widget( load_file_dlg.self, "cpu_opt_frame"); 
	load_file_dlg.asmblr.frame =   (GtkFrame *)
			lookup_widget( load_file_dlg.self, "asm_opt_frame"); 
	load_file_dlg.hll.frame =   (GtkFrame *)
			lookup_widget( load_file_dlg.self, "hll_opt_frame");
	/* boxes */
	load_file_dlg.disasm.box =   (GtkVBox *)
			lookup_widget( load_file_dlg.self, "disasm_meth_vbox");
	load_file_dlg.format.box =   (GtkVBox *) 
			lookup_widget( load_file_dlg.self, "format_opt_vbox"); 
	load_file_dlg.cpu.box =    (GtkVBox *)
			lookup_widget( load_file_dlg.self, "cpu_opt_vbox"); 
	load_file_dlg.asmblr.box =    (GtkVBox *)
			lookup_widget( load_file_dlg.self, "asm_opt_vbox"); 
	load_file_dlg.hll.box =   (GtkVBox *) 
			lookup_widget( load_file_dlg.self, "hll_opt_vbox");	
	
	 	
	/* build glists */
	load_file_dlg.format.list =  make_glist_for_ext( EXT_FORMAT);
	load_file_dlg.cpu.list = make_glist_for_ext( EXT_ARCH);
	load_file_dlg.asmblr.list = make_glist_for_ext( EXT_ASM);
	load_file_dlg.hll.list = make_glist_for_ext( EXT_LANG);
	
	/* set glist for each combo */
	gtk_combo_set_popdown_strings(
			GTK_COMBO( lookup_widget(load_file_dlg.self, "lfd_format_combo") ), 
			load_file_dlg.format.list);
	gtk_combo_set_popdown_strings(
			GTK_COMBO( lookup_widget(load_file_dlg.self, "lfd_cpu_combo") ), 
			load_file_dlg.cpu.list);
	gtk_combo_set_popdown_strings(
			GTK_COMBO( lookup_widget(load_file_dlg.self, "lfd_asm_combo") ), 
			load_file_dlg.asmblr.list);
	gtk_combo_set_popdown_strings(
			GTK_COMBO( lookup_widget(load_file_dlg.self, "lfd_hll_combo") ), 
			load_file_dlg.hll.list);	
	
	/* create disassembly options */	
	sprintf(default_target, "%s/plugins", env_get_home());
	create_disasm_dlg(default_target);

	return(1);
}

void delete_child_control( GtkWidget *widget, gpointer data){
	//printf("delete %s", gtk_widget_get_name(GTK_WIDGET(widget)));
	//printf(" from %s\n", gtk_widget_get_name(GTK_WIDGET(data)));
	gtk_container_remove(data, widget);
	printf("done\n");
	return;	
}

int create_dlg_from_datfile(char *filename, struct lfd_child *c){
	FILE *f;
	GtkWidget *widget;
	GSList *group = NULL;
	char w, text[256], *vboxname;
	int val;
	
	/* first, delete previous vbox */
	vboxname = gtk_widget_get_name((GtkWidget *)c->box);
	if (c->box) {		
		gtk_widget_hide((GtkWidget *) c->box );
		c->box->box.children = NULL;					/* why is this necessary? */
		gtk_object_destroy(GTK_OBJECT(c->box));
		c->frame->bin.child = NULL;						/* and this? */
		gtk_widget_hide((GtkWidget *)c->frame);
		gtk_widget_show((GtkWidget *)c->frame);
	}
	
	/* now, make a new one */
	c->box = (GtkVBox *) gtk_vbox_new(FALSE, 0);
   gtk_widget_set_name(GTK_WIDGET(c->box), vboxname);

   gtk_widget_show (GTK_WIDGET(c->box));
	gtk_container_add (GTK_CONTAINER (c->frame), GTK_WIDGET(c->box));
	
	f = fopen(filename, "r");
	if (!f) {
		
		/* add a label saying "no options for this extension" */
		widget = gtk_label_new("No options provided for this extension");
		gtk_box_pack_start((GtkBox *)c->box, widget, TRUE, FALSE, 0);
		gtk_widget_show(widget);
		
		return(0);
	}
	
	while ( fscanf(f, "%[^|]|%c|%x\n", text, &w, &val) > 0 ){
		switch (w){
			case 'r':
				/* TODO : add support for >1 radio groups */
				/* maybe set group to NULL after 'c' and 's' [separator] */
				widget = gtk_radio_button_new_with_label(
								group, text);
				if (! group) 
					group = gtk_radio_button_group(GTK_RADIO_BUTTON(widget));
				break;
			case 'c':
			default:
				widget = gtk_check_button_new_with_label(text);
		}
		/* save data somewhere */
		gtk_object_set_data((GtkObject *)widget, "value", (void *)val);
		 		
		/*show widget */
		gtk_box_pack_start((GtkBox *)c->box, widget, FALSE, FALSE, 0);
		gtk_widget_ref(widget);
		gtk_widget_show(widget);
				
	}
		
	return(1);
}

int create_disasm_dlg(char *dir){
	char path[PATH_MAX], *buf,  *start;
	glob_t globbuf;
	int len, x;
	GtkRadioButton *rbutton = NULL;
	GSList *rgroup = NULL;
	
	/* find all .bc extensions */
	sprintf(path, "%s/disasm*.bc", dir);
	//printf("%s\n", path);
	glob(path, 0, NULL, &globbuf);
	for (x=0; x<globbuf.gl_pathc; x++){
		start = strstr(globbuf.gl_pathv[x], "/disasm.") + 8;
		len = strstr(globbuf.gl_pathv[x], ".bc") - start;
		buf = malloc(len + 1);
		strncpy( buf, start, len );
		buf[len] = '\0';
		
		rbutton = (GtkRadioButton *)gtk_radio_button_new_with_label(rgroup, buf);
		if (! rgroup) 
			rgroup = gtk_radio_button_group(GTK_RADIO_BUTTON(rbutton));	
		
		gtk_object_set_data((GtkObject *)rbutton, "value", buf);
		gtk_box_pack_start((GtkBox *)load_file_dlg.disasm.box, 
				             (GtkWidget *)rbutton, TRUE, TRUE, 0);
		gtk_widget_show((GtkWidget *)rbutton);		
		
		if (!strcmp(buf, "full"))  
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rbutton), 1);
	}
	globfree(&globbuf);
	
	/* find all .so extensions */
	sprintf(path, "%s/libdisasm.*.so", dir);
	glob(path, 0, NULL, &globbuf);
	for (x=0; x<globbuf.gl_pathc; x++){
		start = strstr(globbuf.gl_pathv[x], "/libdisasm.") + 11;
		len = strstr(globbuf.gl_pathv[x], ".so") - start;
		buf = malloc(len + 1);
		strncpy( buf, start, len );
		buf[len] = '\0';
		
		rbutton = (GtkRadioButton *)gtk_radio_button_new_with_label(rgroup, buf);
		if (! rgroup) 
			rgroup = gtk_radio_button_group(GTK_RADIO_BUTTON(rbutton));	
		
		gtk_object_set_data((GtkObject *)rbutton, "value", buf);
		gtk_box_pack_start((GtkBox *)load_file_dlg.disasm.box, 
				             (GtkWidget *)rbutton, TRUE, TRUE, 0);
		gtk_widget_show((GtkWidget *)rbutton);		
		
		if (!strcmp(buf, "full"))  
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rbutton), 1);
	}
		
	globfree(&globbuf);
	return(1);
}
	


void init_ui_main(GtkWidget *w){
	/* this is called during main() to do all GUI init stuff */
	sob_main_window = w;
	init_db_tree();		   /* move into ON_CREATE CB for tree widget */
	init_asm_win();         /* move into ON_CREATE CB for text widget */
}
